@@ -1219,9 +1219,12 @@ arithmetic_operator_run(struct arithmeticparams *p, int operator,
 {
   size_t i;
   unsigned int numop;
+  int flags = GAL_ARITHMETIC_FLAGS_BASIC;
   gal_data_t *d1=NULL, *d2=NULL, *d3=NULL;
-  int flags = ( GAL_ARITHMETIC_INPLACE | GAL_ARITHMETIC_FREE
-                | GAL_ARITHMETIC_NUMOK );
+
+  /* Set the operating-mode flags if necessary. */
+  if(p->cp.quiet) flags |= GAL_ARITHMETIC_FLAG_QUIET;
+  if(p->envseed)  flags |= GAL_ARITHMETIC_FLAG_ENVSEED;
 
   /* When 'num_operands!=0', the operator is in the library. */
   if(num_operands)
@@ -1409,7 +1412,7 @@ reversepolish(struct arithmeticparams *p)
       else if( !strncmp(token->v, GAL_ARITHMETIC_SET_PREFIX,
                         GAL_ARITHMETIC_SET_PREFIX_LENGTH) )
         gal_arithmetic_set_name(&p->setprm, token->v);
-      else if(    gal_array_name_recognized(token->v)
+      else if(    gal_array_file_recognized(token->v)
                || gal_arithmetic_set_is_name(p->setprm.named, token->v) )
         operands_add(p, token->v, NULL);
       else if( (data=gal_data_copy_string_to_number(token->v)) )
@@ -1457,7 +1460,7 @@ reversepolish(struct arithmeticparams *p)
       /* Read the desired image and report it if necessary. */
       hdu=p->operands->hdu;
       filename=p->operands->filename;
-      if( gal_fits_name_is_fits(filename) )
+      if( gal_fits_file_recognized(filename) )
         {
           /* Read the data, note that the WCS has already been set. */
           p->operands->data=gal_array_read_one_ch(filename, hdu, NULL,
