@@ -5,7 +5,7 @@ Match is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2020, Free Software Foundation, Inc.
+Copyright (C) 2017-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -163,19 +163,21 @@ parse_opt(int key, char *arg, struct argp_state *state)
   /* Set the key to this option. */
   switch(key)
     {
-
-    /* Read the non-option tokens (arguments): */
+    /* Read the non-option tokens (arguments). */
     case ARGP_KEY_ARG:
+      /* The user may give a shell variable that is empty! In that case
+         'arg' will be an empty string! We don't want to account for such
+         cases (and give a clear error that no input has been given). */
       if(p->input1name)
         {
           if(p->input2name)
-            argp_error(state, "only two arguments (input files) should be "
-                       "given, not any more");
+            argp_error(state, "only two arguments (input files) "
+                       "should be given, not any more");
           else
-            p->input2name=arg;
+            { if(arg[0]!='\0') p->input2name=arg; }
         }
       else
-        p->input1name=arg;
+        { if(arg[0]!='\0') p->input1name=arg; }
       break;
 
 
@@ -239,9 +241,17 @@ ui_check_options_and_arguments(struct matchparams *p)
         error(EXIT_FAILURE, 0, "only one argument can be given with the "
               "'--coord' option");
 
-      /* No need for 'p->input2name' or 'p->ccol2'. */
-      gal_data_free(p->ccol2);
-      p->ccol2=NULL;
+      /* In case '--ccol2' is given. */
+      if(p->ccol2 && p->cp.quiet==0)
+        {
+          /* Print a warning to let the user know the option will not be
+             used (the user had probably confused things if they have given
+             it). */
+          error(EXIT_SUCCESS, 0, "WARNING: with '--coord' you only need "
+                "to set '--ccol1' (for the table), '--ccol2' (for the "
+                "'--coord') is not needed because you are directly "
+                "giving the coordinates in any order you want");
+        }
     }
 
   /* '--coord' is not given. */
@@ -267,7 +277,7 @@ ui_check_options_and_arguments(struct matchparams *p)
 
   /* If the first input is a FITS file, make sure its HDU is also given. */
   if( p->input1name
-      && gal_fits_name_is_fits(p->input1name)
+      && gal_fits_file_recognized(p->input1name)
       && p->cp.hdu==NULL )
     error(EXIT_FAILURE, 0, "no HDU for first input. When the input is "
           "a FITS file, a HDU must also be specified, you can use the "
@@ -275,7 +285,7 @@ ui_check_options_and_arguments(struct matchparams *p)
           "from zero), extension name, or anything acceptable by "
           "CFITSIO");
   if( p->input2name
-      && gal_fits_name_is_fits(p->input2name)
+      && gal_fits_file_recognized(p->input2name)
       && p->hdu2==NULL )
     error(EXIT_FAILURE, 0, "no HDU for second input. Please use "
           "the '--hdu2' ('-H') option and give it the HDU number "
@@ -316,7 +326,7 @@ ui_set_mode(struct matchparams *p)
      'NULL'), then we go into catalog mode because currently we assume
      standard input is only for plain text and WCS matching is not defined
      on plain text. */
-  if( p->input1name && gal_fits_name_is_fits(p->input1name) )
+  if( p->input1name && gal_fits_file_recognized(p->input1name) )
     {
       tin1=gal_fits_hdu_format(p->input1name, p->cp.hdu);
       p->mode = (tin1 == IMAGE_HDU) ? MATCH_MODE_WCS : MATCH_MODE_CATALOG;
@@ -340,7 +350,7 @@ ui_set_mode(struct matchparams *p)
      file.*/
   if(p->input2name)
     {
-      if(gal_fits_name_is_fits(p->input2name))
+      if(gal_fits_file_recognized(p->input2name))
         {
           tin2=gal_fits_hdu_format(p->input2name, p->hdu2);
           if(tin1==IMAGE_HDU && tin2!=IMAGE_HDU)
