@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2020, Free Software Foundation, Inc.
+Copyright (C) 2016-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -158,6 +158,7 @@ enum option_keys_enum
   UI_KEY_MAXIMUM,
   UI_KEY_CLUMPSMAGNITUDE,
   UI_KEY_UPPERLIMIT,
+  UI_KEY_UPPERLIMITSB,
   UI_KEY_UPPERLIMITONESIGMA,
   UI_KEY_UPPERLIMITSIGMA,
   UI_KEY_UPPERLIMITQUANTILE,
@@ -174,13 +175,14 @@ enum option_keys_enum
   UI_KEY_GEOSEMIMINOR,
   UI_KEY_GEOAXISRATIO,
   UI_KEY_GEOPOSITIONANGLE,
-  UI_KEY_FWHMOBS,
+  UI_KEY_FWHM,
   UI_KEY_HALFMAXAREA,
+  UI_KEY_HALFMAXRADIUS,
   UI_KEY_HALFMAXSUM,
   UI_KEY_HALFMAXSB,
   UI_KEY_HALFSUMAREA,
   UI_KEY_HALFSUMSB,
-  UI_KEY_HALFRADIUSOBS,
+  UI_KEY_HALFSUMRADIUS,
   UI_KEY_FRACMAXSUM1,
   UI_KEY_FRACMAXSUM2,
   UI_KEY_FRACMAXAREA1,
