@@ -133,8 +133,8 @@ table_selection_range(struct tableparams *p, gal_data_t *col)
 {
   size_t one=1;
   double *darr;
-  int numok=GAL_ARITHMETIC_NUMOK;
-  int inplace=GAL_ARITHMETIC_INPLACE;
+  int numok=GAL_ARITHMETIC_FLAG_NUMOK;
+  int inplace=GAL_ARITHMETIC_FLAG_INPLACE;
   gal_data_t *min=NULL, *max=NULL, *tmp, *ltmin, *gemax=NULL;
 
   /* First, make sure everything is OK. */
@@ -304,8 +304,8 @@ table_selection_equal_or_notequal(struct tableparams *p, gal_data_t *col,
   void *varr;
   char **strarr;
   size_t i, one=1;
-  int numok=GAL_ARITHMETIC_NUMOK;
-  int inplace=GAL_ARITHMETIC_INPLACE;
+  int numok=GAL_ARITHMETIC_FLAG_NUMOK;
+  int inplace=GAL_ARITHMETIC_FLAG_INPLACE;
   gal_data_t *eq, *out=NULL, *value=NULL;
   gal_data_t *arg = e0n1 ? p->notequal : p->equal;
 
@@ -395,9 +395,13 @@ table_select_by_value(struct tableparams *p)
   struct list_select *tmp;
   uint8_t *u, *uf, *ustart;
   size_t i, *s, ngood=0;
-  int inplace=GAL_ARITHMETIC_INPLACE;
+  int inplace=GAL_ARITHMETIC_FLAG_INPLACE;
   gal_data_t *mask, *blmask, *addmask=NULL;
 
+  /* It may happen that the input table is empty! In such cases, just
+     return and don't bother with this step. */
+  if(p->table->dsize==NULL) return;
+
   /* Allocate datasets for the necessary numbers and write them in. */
   mask=gal_data_alloc(NULL, GAL_TYPE_UINT8, 1, p->table->dsize, NULL, 1,
                       p->cp.minmapsize, p->cp.quietmmap, NULL, NULL, NULL);
@@ -759,7 +763,7 @@ table_catcolumn(struct tableparams *p)
   for(filell=p->catcolumnfile; filell!=NULL; filell=filell->next)
     {
       /* Set the HDU (not necessary for non-FITS tables). */
-      if(gal_fits_name_is_fits(filell->v))
+      if(gal_fits_file_recognized(filell->v))
         {
           if(hdull) { hdu=hdull->v; hdull=hdull->next; }
           else
