@@ -5,7 +5,7 @@ Warp is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+Copyright (C) 2015-2021, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -31,6 +31,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/wcs.h>
 #include <gnuastro/fits.h>
 #include <gnuastro/polygon.h>
+#include <gnuastro/pointer.h>
 
 #include "main.h"
 #include "warp.h"
@@ -340,8 +341,8 @@ warp_preparations(struct warpparams *p)
      but in that scenario, we don't want this feature! */
   if( p->centeroncorner==0 && (matrix[2]!=0.0f || matrix[5]!=0.0f) )
     {
-      dsize[1] += abs(matrix[2])+1;
-      dsize[0] += abs(matrix[5])+1;
+      dsize[1] += abs( (int)(matrix[2]) )+1; /* (int): avoid warnings. */
+      dsize[0] += abs( (int)(matrix[5]) )+1;
       if(xmin>0) p->outfpixval[0]=0;
       if(ymin>0) p->outfpixval[1]=0;
     }
@@ -520,6 +521,7 @@ warp(struct warpparams *p)
 {
   int err;
   pthread_t t;          /* All thread ids saved in this, not used. */
+  char *mmapname;
   pthread_attr_t attr;
   pthread_barrier_t b;
   struct iwpparams *iwp;
@@ -540,7 +542,9 @@ warp(struct warpparams *p)
 
 
   /* Distribute the output pixels into the threads: */
-  gal_threads_dist_in_threads(p->output->size, nt, &indexs, &thrdcols);
+  mmapname=gal_threads_dist_in_threads(p->output->size, nt,
+                                       p->cp.minmapsize, p->cp.quietmmap,
+                                       &indexs, &thrdcols);
 
 
   /* Start the warp. */
@@ -586,8 +590,9 @@ warp(struct warpparams *p)
     printf(" Output: %s\n", p->cp.output);
 
 
-  /* Free the allocated spaces: */
+  /* Free the allocated spaces, note that 'indexs' may be memory-mapped. */
   free(iwp);
-  free(indexs);
   gal_data_free(p->output);
+  if(mmapname) gal_pointer_mmap_free(&mmapname, p->cp.quietmmap);
+  else         free(indexs);
 }
