@@ -60,14 +60,16 @@ __BEGIN_C_DECLS  /* From C++ preparations */
 
 
 
-/* Arithmetic flags. */
-#define GAL_ARITHMETIC_INPLACE  0x1
-#define GAL_ARITHMETIC_FREE     0x2
-#define GAL_ARITHMETIC_NUMOK    0x4
+/* Arithmetic on-off bit flags (have to be powers of 2).  */
+#define GAL_ARITHMETIC_FLAG_INPLACE  1
+#define GAL_ARITHMETIC_FLAG_FREE     2
+#define GAL_ARITHMETIC_FLAG_NUMOK    4
+#define GAL_ARITHMETIC_FLAG_ENVSEED  8
+#define GAL_ARITHMETIC_FLAG_QUIET    16
 
-#define GAL_ARITHMETIC_FLAGS_ALL ( GAL_ARITHMETIC_INPLACE        \
-                                   | GAL_ARITHMETIC_FREE         \
-                                   | GAL_ARITHMETIC_NUMOK )
+#define GAL_ARITHMETIC_FLAGS_BASIC ( GAL_ARITHMETIC_FLAG_INPLACE   \
+                                     | GAL_ARITHMETIC_FLAG_FREE    \
+                                     | GAL_ARITHMETIC_FLAG_NUMOK )
 
 
 /* Identifiers for each operator. */
@@ -125,6 +127,7 @@ enum gal_arithmetic_operators
   GAL_ARITHMETIC_OP_DEGREE_TO_RA, /* right ascension to decimal. */
   GAL_ARITHMETIC_OP_DEGREE_TO_DEC,/* declination to decimal. */
   GAL_ARITHMETIC_OP_COUNTS_TO_MAG,/* Counts to magnitude. */
+  GAL_ARITHMETIC_OP_MAG_TO_COUNTS,/* Magnitude to counts. */
   GAL_ARITHMETIC_OP_COUNTS_TO_JY, /* Counts to Janskys with AB-mag zeropoint. */
 
   GAL_ARITHMETIC_OP_MINVAL,       /* Minimum value of array.               */
@@ -148,6 +151,10 @@ enum gal_arithmetic_operators
   GAL_ARITHMETIC_OP_SIGCLIP_MEDIAN,/* Sigma-clipped median of mult. arrays.*/
   GAL_ARITHMETIC_OP_SIGCLIP_STD,  /* Sigma-clipped STD of multiple arrays. */
 
+  GAL_ARITHMETIC_OP_MKNOISE_SIGMA,/* Fixed-sigma noise to every element.   */
+  GAL_ARITHMETIC_OP_MKNOISE_POISSON,/* Poission noise on every element.    */
+  GAL_ARITHMETIC_OP_MKNOISE_UNIFORM,/* Uniform noise on every element.     */
+
   GAL_ARITHMETIC_OP_SIZE,         /* Size of the dataset along an axis     */
 
   GAL_ARITHMETIC_OP_TO_UINT8,     /* Convert to uint8_t.                   */
